#include <afxwin.h>
#include "ids.h"
#include "docview.h"

CApp App; // instantiate the application

IMPLEMENT_DYNCREATE(CMainWin, CFrameWnd)
IMPLEMENT_DYNCREATE(CSampleView, CView)
IMPLEMENT_DYNCREATE(CSampleDoc, CDocument)

// This is the application's message map.
BEGIN_MESSAGE_MAP(CApp, CWinApp)
  ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
  ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
END_MESSAGE_MAP()

// This is the view's message map.
BEGIN_MESSAGE_MAP(CSampleView, CView)
  ON_WM_LBUTTONDOWN()
END_MESSAGE_MAP()

// This is the document's message map.
BEGIN_MESSAGE_MAP(CSampleDoc, CDocument)
  ON_COMMAND(IDM_UNDO, OnUndo)
  ON_COMMAND(IDM_RESET, OnReset)
  ON_COMMAND(IDM_HELP, OnHelp)
END_MESSAGE_MAP()

// This is the frame window's message map.
BEGIN_MESSAGE_MAP(CMainWin, CFrameWnd)
  // not used by this example
END_MESSAGE_MAP()

// Initialize the main window.
CMainWin::CMainWin()
{
  // Not used by this example.
}

// Initialize the application.
BOOL CApp::InitInstance()
{
  CSingleDocTemplate *DocPtr = new CSingleDocTemplate(
    IDR_FRAMEWIN,
    RUNTIME_CLASS (CSampleDoc),
    RUNTIME_CLASS(CMainWin),
    RUNTIME_CLASS(CSampleView)
  );

  AddDocTemplate(DocPtr);

  // register application in registry
  EnableShellOpen();
  RegisterShellFileTypes();

  CCommandLineInfo CLInfo;
  ParseCommandLine(CLInfo);

  if(!ProcessShellCommand(CLInfo)) return FALSE;

  return TRUE;
}

// Initialize the document.
CSampleDoc::CSampleDoc()
{
  m_LocIndex = 0;
}

// Process IDM_HELP -- this is only placeholder help.
afx_msg void CSampleDoc::OnHelp()
{
   App.m_pMainWnd->MessageBox("Document/View Demo",
                              "Placeholder Help");
}

// Handle new document request.
BOOL CSampleDoc::OnNewDocument()
{
  if(!CDocument::OnNewDocument()) return FALSE;

  m_LocIndex = 0;

  return TRUE;
}

// Load or store document.
void CSampleDoc::Serialize(CArchive &arch)
{
  int i;

  if(arch.IsLoading()) {
    arch >> m_LocIndex;
    for(i=0; i<m_LocIndex; i++)  arch >> m_Locations[i]; 
  }
  else {
    arch << m_LocIndex;
    for(i=0; i<m_LocIndex; i++)  arch << m_Locations[i]; 
  }
}

// Undo last line.
afx_msg void CSampleDoc::OnUndo()
{
  m_LocIndex--;
  if(m_LocIndex<0) m_LocIndex = 0;

  SetModifiedFlag();
  UpdateAllViews(NULL);
}

// Reset point index and erase window.
afx_msg void CSampleDoc::OnReset()
{
  m_LocIndex = 0;

  SetModifiedFlag();
  UpdateAllViews(NULL);
}

// Record position of mouse click.
afx_msg void CSampleView::OnLButtonDown(UINT flags, CPoint loc)
{
  CSampleDoc *DocPtr = (CSampleDoc *) GetDocument();

  DocPtr->m_Locations[DocPtr->m_LocIndex] = loc;
  DocPtr->m_LocIndex++;
  if(DocPtr->m_LocIndex == MAXLOC) {
    DocPtr->m_LocIndex = MAXLOC;
    App.m_pMainWnd->MessageBox("Too Many Points", "Error");
  }
  DocPtr->SetModifiedFlag();
  DocPtr->UpdateAllViews(NULL);
}

// Draw lines.
void CSampleView::OnDraw(CDC *DC)
{
  CSampleDoc *DocPtr = (CSampleDoc *) GetDocument();
  int i;

  for(i=0; i<DocPtr->m_LocIndex; i++)
    DC->LineTo(DocPtr->m_Locations[i]);
}